(function($) {
/**
 * @class Drupal.availabilityCalendar.Viewport
 *
 * @constructor Creates a new Drupal.availabilityCalendar.Viewport object.
 * @param Integer cid
 *   Required: the cid of the calendar to operate on.
 * @param object settings
 *   Optional: object with the following properties
 *   (name type (default) description):
 *   {
 *     month
 *       width     int (0) the width of 1 calendar month, used to calculate the width of the
 *                         viewport. If 0, use CSS to define the width of the viewport.
 *       height    int (0) the height of 1 calendar month, used to calculate the height of the
 *                         viewport. If 0, use CSS to define the height of the viewport.
 *     cols        int (3) The number of months that is shown horizontally in the viewport.
 *     rows        int (2) The number of months that is shown vertically in the viewport.
 *     scroll      int (0) indicating how many rows or cols to scroll.
 *     animate     Animation parameters, @see http://api.jquery.com/animate/
 *       backward  Advanced usage, do not pass in.
 *       forward   Advanced usage, do not pass in.
 *       speed     int|string ('slow') The animation speed, see jquery documentation of animate().
 *     totalMonths int (calculated) Advanced usage, do not pass in.
 *     firstMonth  int (calculated) Advanced usage, do not pass in.
 *   }
 */
Drupal.availabilityCalendar.Viewport = function(cid, settings) {
  var calendar = Drupal.availabilityCalendar.get(cid);
  var buttonBack = null;
  var buttonForward = null;
  var viewport = null;
  _init();

  /**
   * Initializes the viewport.
   */
  function _init() {
    _initSettings();
    _initDimensions();
    _initHandlers();
  };

  /**
   * Extends the settings with their defaults and adds the animation instructions.
   */
  function _initSettings() {
    settings = $.extend(true, {
      month: {width: 0, height: 0},
      cols: 3,
      rows: 2,
      scroll: 1,
      animate: {speed: "slow"},
      totalMonths: calendar.getNumberOfMonths(),
      firstMonth: 1
    }, settings);

    if (settings.month.width == 0) {
      // Get outer width of a month wrapper
      settings.month.width = $(".cal-month", calendar.getCalendar()).outerWidth(true);
      if (settings.month.width == 0) {
        // @todo: calendar is hidden: get outer width differently
        settings.month.width = $(".cal-month", calendar.getCalendar()).outerWidth(true);
      }
    }
    if (settings.month.height == 0) {
      // Get outer width of a month wrapper.
      settings.month.height = $(".cal-month", calendar.getCalendar()).outerHeight(true);
      if (settings.month.height == 0) {
        // @todo: Calendar is hidden: get outer width differently.
        settings.month.height = $(".cal-month", calendar.getCalendar()).outerHeight(true);
      }
    }

    if (!settings.animate.backward) {
      settings.animate.backward = settings.rows > 1
        ? { top: "+=" + (settings.scroll * settings.month.height) }
        : { left: "+=" + (settings.scroll * settings.month.width) };
    };
    if (!settings.animate.forward) {
      settings.animate.forward = settings.rows > 1
        ? { top: "-=" + (settings.scroll * settings.month.height) }
        : { left: "-=" + (settings.scroll * settings.month.width) };
    };
  }

  /**
   * Sets the dimensions of the viewport.
   */
  function _initDimensions() {
    viewport = $(".cal-viewport", calendar.getCalendar());
    if (settings.month.width != 0) {
      viewport.width(settings.cols * settings.month.width);
    }
    if (settings.month.height != 0) {
      viewport.height(settings.rows * settings.month.height);
    }
    // Move to the inner viewport as that is the element to be animated.
    viewport = viewport.children();
    // If scrolling horizontally, we want the inner viewport to be infinitely wide.
    if (settings.rows == 1) {
      viewport.width(10000);
    }
  }

  /**
   * Initialize event handlers for our own buttons.
   */
  function _initHandlers() {
    buttonBack = $(".cal-backward", calendar.getCalendar());
    buttonForward = $(".cal-forward", calendar.getCalendar());
    buttonBack.click(function() { scrollBackward(); });
    buttonForward.click(function() { scrollForward(); });
    setEnabledState();
  }

  /**
   * Set the enabled/disabled state of the clickable elements.
   *
   * This function uses the disabled attribute. Although officially meant for
   * form elements (probably buttons in our cases), it can be used on <a> tags
   * as well. In combination with the CSS properties cursor and pointer-events
   * (CSS3) you can visibly/effectively disable an <a> tag:
   * a[disabled] { cursor: auto; pointer-events: none }
   */
  function setEnabledState() {
    if (settings.firstMonth <= 1) {
      buttonBack.attr("disabled", "disabled");
    }
    else {
      buttonBack.removeAttr("disabled");
    }
    if (settings.firstMonth + settings.rows * settings.cols > settings.totalMonths) {
      buttonForward.attr("disabled", "disabled");
    }
    else {
      buttonForward.removeAttr("disabled");
    }
  }

  /**
   * Scroll the viewport backward (if possible).
   */
  function scrollBackward() {
    if (settings.firstMonth > 1) {
      viewport.animate(settings.animate.backward, settings.animate.speed);
      settings.firstMonth -= settings.rows > 1 ? settings.scroll * settings.cols : settings.scroll;
      setEnabledState();
    }
  };

  /**
   * Scroll the viewport forward (if possible).
   */
  function scrollForward() {
    if (settings.firstMonth + settings.rows * settings.cols <= settings.totalMonths) {
      viewport.animate(settings.animate.forward, settings.animate.speed);
      settings.firstMonth += settings.rows > 1 ? settings.scroll * settings.cols : settings.scroll;
      setEnabledState();
    }
  };

  return {
    // Publicly exposed methods for those who define their own buttons.
    scrollBackward: scrollBackward,
    scrollForward: scrollForward
  };
};
})(jQuery);
